@@ -1,15 +1,17 @@
 from abc import ABC, abstractmethod
-from contextlib import contextmanager
+from contextlib import contextmanager, redirect_stdout
 from copy import copy, deepcopy
 from email.mime.text import MIMEText
 from functools import wraps, partial
 import inspect
 import logging
 import os
+from pathlib import Path
 import signal
 import sys
 import time
 import warnings
+from weakref import WeakSet
 
 from htools import hdir
 
@@ -575,7 +577,7 @@ class ReadOnly:
     Examples
     --------
     class Dog:
-        breed = ReadOnly('breed')
+        breed = ReadOnly()
         def __init__(self, breed, age):
             # Once breed is set in the line below, it cannot be changed.
             self.breed = breed
@@ -595,9 +597,12 @@ class ReadOnly:
     PermissionError: Attribute is read-only.
     """
 
-    def __init__(self, name):
+    def __init__(self):
+        self.name = None
+        self.initialized = WeakSet()
+
+    def __set_name__(self, owner, name):
         self.name = name
-        self.initialized = set()
 
     def __get__(self, instance, cls):
         if instance is None:
@@ -965,6 +970,49 @@ def debug(func=None, prefix='', arguments=True):
     return wrapper
 
 
+def log_stdout(func=None, fname=''):
+    """Decorator that logs all stdout produced by a function.
+
+    Parameters
+    ----------
+    func: function
+        If the decorator is used without parenthesis, the function will be
+        passed in as the first argument. You never need to explicitly specify
+        a function.
+    fname: str
+        Path to log file which will be created. If None is specified, the
+        default is to write to ./logs/wrapped_func_name.log. If specified,
+        this must be a keyword argument.
+
+    Examples
+    --------
+    @log_stdout
+    def foo(a, b=3):
+        print(a)
+        a *= b
+        print(a)
+        return a**b
+
+    @log_stdout(fname='../data/mylog.log')
+    def foo(a, b=3):
+        ...
+    """
+    if not func:
+        return partial(log_stdout, fname=Path(fname))
+    if not fname:
+        fname = Path(f'./logs/{func.__name__}.log')
+
+    @wraps(func)
+    def wrapper(*args, **kwargs):
+        os.makedirs(fname.parent, exist_ok=True)
+        with open(fname, 'w') as f:
+            with redirect_stdout(f):
+                out = func(*args, **kwargs)
+        return out
+
+    return wrapper
+
+
 def wrapmethods(*decorators, magics=False, internals=False):
     """Class wrapper that applies 1 or more decorators to every non-magic
     method. For example, we often want @debug to be applied to many different
@@ -1016,6 +1064,57 @@ def wrapmethods(*decorators, magics=False, internals=False):
     return wrapper
 
 
+def delegate(attr):
+    """Decorator that automatically delegates attribute calls to an attribute
+    of the class. This is a nice convenience to have when using composition.
+    This does NOT affect magic methods; for that, see the `forwardable`
+    library.
+
+    Note: I suspect this could lead to some unexpected behavior so be careful
+    using this in production.
+
+    Parameters
+    ----------
+    attr: str
+        Name of variable to delegate to.
+
+    Examples
+    --------
+    Example 1: We can use BeautifulSoup methods like `find_all` directly on
+    the Page object. Most IDEs should let us view quick documentation as well.
+
+    @delegate('soup')
+    class Page:
+        def __init__(self, url, logfile, timeout):
+            self.soup = self.fetch(url, timeout=timeout)
+        ...
+
+    page = Page('http://www.coursera.org')
+    page.find_all('div')
+
+    Example 2: Magic methods are not delegated.
+
+    @delegate('data')
+    class Foo:
+        def __init__(self, data, city):
+            self.data = data
+            self.city = city
+
+    >>> f = Foo(['a', 'b', 'c'], 'San Francisco')
+    >>> len(f)
+
+    TypeError: object of type 'Foo' has no len()
+    """
+    def wrapper(cls):
+        def f(self, new_attr):
+            delegate = getattr(self, attr)
+            return getattr(delegate, new_attr)
+        cls.__getattr__ = f
+        return cls
+
+    return wrapper
+
+
 def add_docstring(func):
     """Add the docstring from another function/class to the decorated
     function/class.
